"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const prettier_1 = require("prettier");
exports.default = async (directory, files, { check, config, onExamineFile, onCheckFile, onWriteFile, } = {}) => {
    for (const relative of files) {
        onExamineFile?.(relative);
        const file = path_1.default.join(directory, relative);
        const options = {
            ...(await (0, prettier_1.resolveConfig)(file, {
                config,
                editorconfig: true,
            })),
            filepath: file,
        };
        const input = fs_1.default.readFileSync(file, 'utf8');
        if (check) {
            const isFormatted = await (0, prettier_1.check)(input, options);
            onCheckFile?.(relative, isFormatted);
            continue;
        }
        const output = await (0, prettier_1.format)(input, options);
        if (output !== input) {
            fs_1.default.writeFileSync(file, output);
            onWriteFile?.(relative);
        }
    }
};
//# sourceMappingURL=processFiles.js.map